# -*- coding: utf-8 -*-

import datetime
import time

import dateparser

from dropshipping.utils.utils import is_asin_valid

from em_tasks import logger
from em_tasks.utils.product_service import ProductService
from em_tasks.store.store_util import StoreUtil
from em_tasks.spree.product_util import ProductUtil
from em_tasks.utils.product_converters import AmzProductConverter
from em_tasks.utils.blacklist_filter import BlacklistFilter
from em_tasks.utils.pipelines.price_calculator import PriceCalculatorPipeline
from em_tasks.utils.pipelines.price_filter import PriceFilterPipeline
from em_tasks.utils.pipelines.image_url_corrector import ImageUrlCorrectorPipeline
from em_tasks.utils.pipelines.amz_products_filter import AmzProductsFilterPipeline
from em_tasks.utils.pipelines.upc_validator import UpcValidatorPipeline
from em_tasks.store.models import *


class AmzUploadableProductsFilter():
  def __init__(self, store_code, from_marketplace, to_marketplace, product_service, offer_service, price_calculator, products_path, output_path, ttl=7):
    self.store = StoreUtil.get_store_by_code(store_code)
    if not self.store:
      return

    credential = json.loads(self.store.api_credential)
    self.product_util = ProductUtil(credential['endpoint'], credential['api_key'], credential.get('api_version', 'v1'))

    self.blacklist_filter = BlacklistFilter()

    self.product_service = product_service
    self.offer_service = offer_service
    self.from_marketplace = from_marketplace.lower()
    self.to_marketplace = to_marketplace.lower()
    self.pc = AmzProductConverter(self.from_marketplace)
    self.indice_name = 'amz_products_api_{}'.format(self.from_marketplace)
    self.condition = 'new'

    self.products_path = products_path

    self.output_path = os.path.abspath(os.path.expanduser(output_path))
    output_dir = os.path.dirname(self.output_path)
    if not os.path.isdir(output_dir):
      os.makedirs(output_dir)
    self.output_fh = open(self.output_path, 'w+', encoding='utf-8', errors='ignore')

    today = datetime.date.today()
    self.expire_date = today - datetime.timedelta(days=ttl)
    self.sales_rank_min = 1000000
    self.weight_max = 2
    self.dimension_max = 12

    self.uploaded_prods = dict()
    self.blacklisted_products = dict()
    self.unuploadable_products = dict()

    self.export = self.to_marketplace == self.from_marketplace
    self.products_filter_pipeline = AmzProductsFilterPipeline(self.from_marketplace, self.weight_max, self.dimension_max, self.sales_rank_min, self.export)
    price_calculator_pipeline = PriceCalculatorPipeline(price_calculator)
    price_filter_pipeline = PriceFilterPipeline()
    image_url_corrector_pipeline = ImageUrlCorrectorPipeline()
    upc_validator_pipeline = UpcValidatorPipeline()
    self.pipelines = [price_calculator_pipeline, price_filter_pipeline, image_url_corrector_pipeline, upc_validator_pipeline]

  def run(self):
    # try:
    #   for prod in self.product_util.get_amz_products(self.from_marketplace):
    #     source_product_id = prod.get('source_product_id', prod.get('asin', None))
    #     if not source_product_id:
    #       continue

    #     self.uploaded_prods[source_product_id] = None

    #   logger.debug('[UploadedProductsLoaded] ProductsCnt: %s', len(self.uploaded_prods))
    # except Exception as e:
    #   logger.exception(e)

    try:
      asins_buf = []
      batch_size = 500
      options = {'size': batch_size}
      with open(self.products_path, encoding='utf-8', errors='ignore') as fh:
        for line in fh:
          asin = line.strip()
          if not asin:
            continue

          if not is_asin_valid(asin):
            continue

          asins_buf.append(asin)
          if len(asins_buf) < batch_size:
            continue

          self.process_products(asins_buf)
          asins_buf = []

        if asins_buf:
          self.process_products(asins_buf)
          asins_buf = []
    finally:
      self.output_fh.close()

  def process_products(self, asins):
    filtered_prods = dict()
    prods = self.product_service.search_products(self.indice_name, asins)
    if not prods:
      logger.error("[ProductServiceError] No products loaded")
      return

    for asin, prod in prods.items():
      result = self.products_filter_pipeline.check_product(prod)
      passed = result.get('passed', True)
      reason = result.get('reason', '')
      if not passed:
        logger.debug('[ProductFiltered] ASIN: %s, Reason: Unuploadable - %s', asin, reason)
        continue

      # result = self.check_blacklist(prod)
      # passed = result.get('passed', True)
      # reason = result.get('reason', '')
      # if not passed:
      #   logger.debug('[ProductFiltered] ASIN: %s, Reason: Blacklisted - %s', asin, reason)
      #   continue

      filtered_prods[asin] = prod

    offers = dict()
    src_offers = None
    max_retries = 3
    while max_retries > 0:
      try:
        src_offers = self.offer_service.get_offers(self.from_marketplace, asins, self.condition)
        if not isinstance(src_offers, dict):
          logger.info('[OfferService] Temporary unavailable! Wait for 10 seconds to retry.')
          max_retries -= 1
          time.sleep(10)
          continue

        break
      except Exception as e:
        logger.exception(e)
        max_retries -= 1
        time.sleep(3)
    if not src_offers:
      return

    products = []
    for asin, product_info in filtered_prods.items():
      prod = self.pc.convert(product_info)

      src_offer = src_offers.get(asin, None)
      if not src_offer:
        logger.debug('[NoOffer] ASIN: %s', asin)
        continue

      offer_time_s = src_offer.get('time', None)
      if offer_time_s:
        offer_time = None
        try:
          offer_time = dateparser.parse(offer_time_s).date()
        except Exception as e:
          logger.exception(e)

        if offer_time:
          if offer_time < self.expire_date:
            logger.debug('[OfferExpired] ASIN: %s, Offer: %s', asin, src_offer)
            prod['price'] = 0
            prod['currency'] = src_offer.get('currency', 'USD')
          else:
            prod['price'] = src_offer.get('price', 0)
            prod['currency'] = src_offer.get('currency', 'USD')
        else:
          prod['price'] = 0
          prod['currency'] = src_offer.get('currency', 'USD')
      else:
        prod['price'] = src_offer.get('price', 0)
        prod['currency'] = src_offer.get('currency', 'USD')
      products.append(prod)

    for prod in products:
      product = prod
      for pipeline in self.pipelines:
        product = pipeline.execute(product)
        if not product:
          break

      if not product or not product.get('price', 0):
        continue

      product["shipping_days_min"] = None
      product["shipping_days_max"] = None
      self.output_fh.write(json.dumps(product, ensure_ascii=False))
      self.output_fh.write("\n")

  def check_blacklist(self, product):
    blacklisted = {'passed': True, 'reason': ''}
    s = ' {} -{} '.format(self.pc.get_brand(product).lower(), product.get('title', '').lower())
    result = self.blacklist_filter.is_blacklisted(s)
    if result:
      blacklisted = {'passed': False, 'reason': '[BlacklistKeyword] {}'.format(result)}

    return blacklisted
