# -*- coding: utf-8 -*-

import re
import datetime

from gspread.exceptions import APIError
from tracking_numbers import get_tracking_number

from em_tasks import logger
from em_tasks.store.store_util import StoreUtil
from em_tasks.google.sheet_api import SheetAPI
from em_tasks.utils.shipment_reader import GSheetShipmentReader
from em_tasks.utils.tracking_parsers import UPSTrackingParser, FedexTrackingParser, USPSTrackingParser
from em_tasks.store.models import *


class ShipmentUpdater():
  def __init__(self, service_account_data, spreadsheet_id, ups_api=None, fedex_api=None, usps_api=None):
    self.service_account_data = service_account_data
    self.spreadsheet_id = spreadsheet_id
    self.sheet_api = SheetAPI(service_account_data)
    self.shipment_reader = GSheetShipmentReader(self.sheet_api)
    self.ups_api = ups_api
    self.fedex_api = fedex_api
    self.usps_api = usps_api
    self.ups_parser = UPSTrackingParser()
    self.fedex_parser = FedexTrackingParser()
    self.usps_parser = USPSTrackingParser()
    self.tracking_pattern = re.compile('[0-9A-Za-z-]{10,100}')

  def run(self):
    for worksheet, headers, records in self.shipment_reader.read(self.spreadsheet_id):
      results = {}
      for record in records:
        tracking_number = record.get('tracking', None)
        if not tracking_number:
          logger.debug('[NoTrackingNumber] %s', record)
          continue

        if not self.tracking_pattern.match(tracking_number):
          logger.debug('[InvalidTracking] %s', record)
          continue

        tracking_number = tracking_number.strip()

        carrier = record.get('carrier', None)
        candidate_carrier = None
        try:
          tn = get_tracking_number(tracking_number)
          if tn and tn.courier:
            candidate_carrier = tn.courier.code.upper()
        except:
          pass

        if not carrier and not candidate_carrier:
          logger.debug('[NoCarrier] %s', record)
          continue

        status = record.get('status', None)
        if status and status.lower() == 'delivered':
          logger.debug('[ShipmentDelivered] %s', record)
          continue

        carrier_upper = carrier.strip().upper()
        if carrier_upper.find('UPS') != -1:
          carrier = 'UPS'
        elif carrier_upper.find('FEDEX') != -1:
          carrier = 'FEDEX'
        elif carrier_upper.find('USPS') != -1:
          carrier = 'USPS'
        else:
          logger.debug('[UnsupportedCarrier] Carrier: %s, Tracking: %s', carrier, tracking_number)
          continue

        if candidate_carrier and carrier != candidate_carrier:
          message = 'Original: {}, Candidate: {}'.format(carrier, candidate_carrier)
          logger.debug(message)
          self.save_shipment_status(tracking_number, {'error': message}, worksheet, headers)
          continue

        if carrier == 'UPS':
          if not self.ups_api:
            logger.debug('[SKIP] Reason: No UPS API')
            continue

          try:
            tracking = self.ups_api.get_tracking_details(tracking_number)
            result = self.ups_parser.parse(tracking_number, tracking)
            if not result:
              logger.info('[NoTrackingResult] %s', tracking_number)
            elif result and 'error' in result:
              logger.info('[TrackingTrackError] %s', result['error'])
              self.save_shipment_status(tracking_number, result, worksheet, headers)
            else:
              self.save_shipment_status(tracking_number, result, worksheet, headers)
              # results[tracking_number] = result
          except Exception as e:
            logger.exception(e)

        if carrier == 'FEDEX':
          if not self.fedex_api:
            logger.debug('[SKIP] Reason: No Fedex API')
            continue

          try:
            tracking = self.fedex_api.get_tracking_details(tracking_number)
            result = self.fedex_parser.parse(tracking_number, tracking)
            if not result:
              logger.info('[NoTrackingResult] %s', tracking_number)
            elif result and 'error' in result:
              logger.info('[TrackingTrackError] %s', result['error'])
              self.save_shipment_status(tracking_number, result, worksheet, headers)
            else:
              self.save_shipment_status(tracking_number, result, worksheet, headers)
              # results[tracking_number] = result
          except Exception as e:
            logger.exception(e)

        if carrier == 'USPS':
          if not self.usps_api:
            logger.debug('[SKIP] Reason: No USPS API')
            continue

          try:
            tracking = self.usps_api.get_tracking_details(tracking_number)
            result = self.usps_parser.parse(tracking_number, tracking)
            if not result:
              logger.info('[NoTrackingResult] %s', tracking_number)
            elif result and 'error' in result:
              logger.info('[TrackingTrackError] %s', result['error'])
              self.save_shipment_status(tracking_number, result, worksheet, headers)
            else:
              self.save_shipment_status(tracking_number, result, worksheet, headers)
              # results[tracking_number] = result
          except Exception as e:
            logger.exception(e)

      # if not results:
      #   continue

      # for tracking_number, result in results.items():
      #   self.save_shipment_status(tracking_number, result, worksheet, headers)

  def save_shipment_status(self, tracking_number, result, worksheet, headers):
    if 'error' in result and result['error']:
      status = result['error']
    elif result['delivered']:
      status = 'Delivered'
    else:
      status = "LastUpdatedDate: {}\nStatus: {}".format(result['last_update_date'], result['last_update_status'])

    tracking_col = headers.index('tracking') + 1
    status_col = headers.index('status') + 1
    status_col_offset = status_col - tracking_col

    cell = self.sheet_api.find_cell_by_key(worksheet, tracking_number)
    row = cell.row
    s_number_col = cell.col + status_col_offset
    self.sheet_api.update_worksheet_cell(worksheet, row, s_number_col, status)
    logger.info('[ShipmentStatusUpdated] %s', result)

