import os
import time
import datetime

from em_tasks.utils.product_converters import AmzProductConverter
from em_tasks.utils.product_loaders.base import BaseProductLoader
from em_tasks import logger


class AmzMongoProductLoader(BaseProductLoader):
  def __init__(self, product_service, offer_service, marketplace, mongo_client):
    self.product_service = product_service
    self.offer_service = offer_service
    self.marketplace = marketplace.lower()
    self.indice_name = 'amz_products_api_{}'.format(self.marketplace)
    self.us_indice_name = 'amz_products_api_us'
    self.condition = 'new'
    self.pc = AmzProductConverter(self.marketplace)
    self.product_ids_path = None
    self.mongo_client = mongo_client

  def save_asins(self, asins):
      with open(f"{self.marketplace}_asin.txt", "w+") as file:
          for asin in asins:
              file.write(f"{asin}\n")

  def load_products_from_mongo(self):

    batch_size = 500
    product_ids_buf = []
    asins = set()
    one_week_ago = datetime.datetime.now() - datetime.timedelta(weeks=1)
    query = {"time": {"$gt": one_week_ago}, "type": "offer"}
    # with self.mongo_client.client[f"amazon_{self.marketplace}"].product_list.find(query, batch_size=4000, no_cursor_timeout=True) as cursor:
    with self.mongo_client.client[f"amazon_{self.marketplace}"].asins_sended.find(query, batch_size=4000, no_cursor_timeout=True) as cursor:
        for doc in cursor:
            asins.add(doc["asin"])
    logger.debug(f"[ASIN Count]: {len(asins)}")
    self.save_asins(asins)
    for asin in asins:
        product_ids_buf.append(asin)
        if len(product_ids_buf) < batch_size:
          continue

        product_ids_buf = self.remove_sold_on_us(product_ids_buf)
        products = self.get_products_by_id(product_ids_buf)
        if not products:
          return

        for product in products:
          yield product

        product_ids_buf = []

    if product_ids_buf:
      product_ids_buf = self.remove_sold_on_us(product_ids_buf)
      products = self.get_products_by_id(product_ids_buf)
      if not products:
        return

      for product in products:
        yield product

      product_ids_buf = []

  def remove_sold_on_us(self, product_ids):
    max_retries = 3
    while max_retries > 0:
      try:
        products = self.product_service.search_products(self.us_indice_name, product_ids)
        if not isinstance(products, dict):
          logger.info('[ProductService] Temporary unavailable! Wait for 10 seconds to retry.')
          max_retries -= 1
          time.sleep(10)
          continue

        for product_id in product_ids:
          if product_id in products and products[product_id]:
            logger.debug('[ProductSoldOnUS] ASIN: %s', product_id)
            try:
              product_ids.remove(product_id)
            except ValueError as e:
                logger.exception('[ASIN does not exist in list] ASIN: %s', product_id)

        return product_ids
      except Exception as e:
        logger.exception(e)
        max_retries -= 1
        time.sleep(3)

  def get_products_by_id(self, product_ids):

    max_retries = 3
    while max_retries > 0:
      try:
        products = self.product_service.search_products(self.indice_name, product_ids)
        if not isinstance(products, dict):
          logger.info('[ProductService] Temporary unavailable! Wait for 10 seconds to retry.')
          max_retries -= 1
          time.sleep(10)
          continue

        for product_id in product_ids:
          if product_id not in products or not products[product_id]:
            logger.debug('[NoProductInformation] ASIN: %s', product_id)

        for product_id, product_info in products.items():
            yield product_id, product_info
        break
      except Exception as e:
        logger.exception(e)
        max_retries -= 1
        time.sleep(3)

  def get_products(self, product_ids):
    src_offers = None
    max_retries = 3
    while max_retries > 0:
      try:
        src_offers = self.offer_service.get_offers(self.marketplace, product_ids, self.condition)
        if not isinstance(src_offers, dict):
          logger.info('[OfferService] Temporary unavailable! Wait for 10 seconds to retry.')
          max_retries -= 1
          time.sleep(10)
          continue

        break
      except Exception as e:
        logger.exception(e)
        max_retries -= 1
        time.sleep(3)
    if not src_offers:
      return

    items = []
    max_retries = 3
    while max_retries > 0:
      try:
        products = self.product_service.search_products(self.indice_name, product_ids)
        if not isinstance(products, dict):
          logger.info('[ProductService] Temporary unavailable! Wait for 10 seconds to retry.')
          max_retries -= 1
          time.sleep(10)
          continue

        for product_id in product_ids:
          if product_id not in products or not products[product_id]:
            logger.debug('[NoProductInformation] ASIN: %s', product_id)

        for product_id, product_info in products.items():
          product = self.pc.convert(product_info)
          src_offer = src_offers.get(product_id, None)
          if not src_offer:
            logger.debug('[NoOffer] ASIN: %s', product_id)
            continue

          product['price'] = src_offer.get('price', 0)
          product['currency'] = src_offer.get('currency', 'USD')

          items.append(product)

        break
      except Exception as e:
        logger.exception(e)
        max_retries -= 1
        time.sleep(3)

    return items
